/*
	This file is part of XFuniter.

	menu.h V2.2.2 - menu routines.

	Copyright (C) 1995-2003 Stijn Wolters.

	See README for contact information.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version. See

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	See COPYING for more information.
*/

#ifndef _MENU
	#define _MENU

	#include <stdio.h>
	#include <stdlib.h>
	#include <ctype.h>
	#include <string.h>
    #include <allegro.h>
	#include "alconio.h"
	#include "bitmanip.h"

	/* Alignment definitions */

	#define	LEFT                	0
	#define CENTER					1
	#define RIGHT					2

	/* General control definitions */

	#define	DEFAULT					15
	#define	SELECTED				14

	#define FOCUS					13
	#define	VISIBLE					12
	#define	ENABLED					11

	#define TITLE					10
	#define	BORDER					 9
	#define HOTKEY					 8
	#define HINT					 7

	/* Window definitions */

	#define	CAN_CLOSE				 6
	#define	CAN_MINIMIZE			 5
	#define CAN_MAXIMIZE			 4
	#define	CAN_RESIZE				 3
	#define CAN_MOVE				 2
	#define	CAN_EDIT				 1

	/*
	**	TEdit definitions:
	**
	**	NUM:	Only numbers (digits).
	**	CHR:	Only characters.
	**	NUMCHR:	Only numbers & chars.
	**	ASCII:	All ASCII signs > 31.
	**	FNUM:	Only numbers, + - . e E.
	*/

	#define NUM                 	 6
	#define CHR                 	 5
	#define NUMCHR              	 4
	#define ASCII               	 3
	#define FNUM               		 2
	#define INUM					 1

	/* List of types */

	#define	TLABEL					 0
	#define TEDIT					 1
	#define TBUTTON					 2
	#define TMENUITEM				 3

	/*
	**	TEdit: Define some macro's for the cursor-control. Range checking is
	**	for the calling-routine.
	*/

	#define CURSOR_UP(X)        gotoxy(wherex(), wherey() - (X))
	#define CURSOR_DOWN(X)      gotoxy(wherex(), wherey() + (X))
	#define CURSOR_LEFT(X)      gotoxy(wherex() - (X), wherey())
	#define CURSOR_RIGHT(X)     gotoxy(wherex() + (X), wherey())

    /* TColor */
    
    typedef int TColor;
    
	/*
	**	TControl:
	**  ---------------------------------------------------------------------
	**
	**	Caption						-	Visible text of the control.
	**
	**	Type						-	Control-type (see defines).
	**
	**	XPos						-	X position of control.
	**	YPos						-	Y position of control.
	**	Height						-	Height of control.
	**	Width						-	Width of control.
	**
	**	Attributes					-	See defines.
	**
	**	AutoFocus					-	Index of control that comes active
	**									when the current control is selected.
	**
	**	Hotkey						-	Key(combination) to active this item.
	**
	**	FgCol, BkCol				-	Forground/Backgroundcolor (standard).
	**	FgSelCol, BkSelCol  		-	Forground/Backgroundcolor (selected).
	**
	**	FgHotkeyCol, BkHotkeyCol	-	Forground/Backgroundcolor (hotkey).
	**
	**	---------------------------------------------------------------------
	*/

	typedef struct {
		char			*Caption;

		unsigned int	Type, XPos, YPos, Height, Width, Align, Attributes,
						AutoFocus, Hotkey;

		TColor			FgCol, BkCol,
						FgSelCol, BkSelCol,
						FgHotkeyCol, BkHotkeyCol;

	} TControl;

	/*
	**	TEvent:
	**	---------------------------------------------------------------------
	**
	**	ControlIndex				-	Index of the control for which the
	**									event occured.
	**
	**	GroupIndex					-	Index of the group for which the
	**									event occured.
	**
	**	KeyPressed					-	Key(combination) that caused the
	**									event.
	**
	**	---------------------------------------------------------------------
	*/


	typedef struct {
		int				ControlIndex, GroupIndex, KeyPressed;
	} TEvent;

	/*
	**	TMenu:
	**	---------------------------------------------------------------------
	**
	**	Caption						-	Header text.
	**
	**	XPos						-	X position of the menu.
	**
	**	YPos						-	Y position of the menu.
	**
	**	Height						-	Height.
	**
	**	Width						-	Width.
	**
	**	Attributes					-	Can Close
	**
	**	NumXControls				-	Number of TControl (X).
	**
	**	NumYControls				-	Number of TControl (Y).
	**
	**	NumControls					- 	Number of TControl (X*Y).
	**
	**	DefaultControl				-	Default highlighted control.
	**
	**	FgCol						-	Forground color.
	**
	**	BkCol						-	Background color.
	**
	**	Control	(struct)			-	TControl (pointer to items).
	**
	**	Event	(struct)			-	TEvent (Events).
	*/

	typedef struct {
		char			*Caption;
		unsigned int	XPos, YPos, Height, Width, Attributes,
						NumXControls, NumYControls, NumControls,
						DefaultControl;
		TColor			FgCol, BkCol;
		TControl		*Item;
		TEvent			Event;
	} TMenu;

	/*
	**	Functions.
	*/

	int getkey(void);

	/* TControl */

	void DrawControl(TControl *Control);
	int SetCaption(TMenu *Menu, TControl *Control, int ItemNum,
		char *NewCaption);

	/* TMenu */

	int CreateMenu(TMenu *Menu, char *Caption, unsigned int XPos,
		unsigned int YPos, unsigned int Height, unsigned int Width,
		unsigned int Attributes, unsigned int NumXControls,
		unsigned int NumYControls, unsigned int DefaultControl,
		TControl *Control, unsigned int FgCol, unsigned int BkCol);
	int DestroyMenu(TMenu *Menu);
	int DrawMenu(TMenu *Menu);

	/* General */

	void Select(TControl *Control);
	void UnSelect(TControl *Control);

	/* TEvent */

	int DoEvent(TMenu *Menu);
	int DoEditEvent(TControl *Control, unsigned int key, int Mode);

#endif
